using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;
using Unity;

namespace JayJayWon
{
    public static class ClothSettingCollection
    {
        const float staticMin = 10f;
        const float staticMax = 10f;

        const float dragMin = 0.01f;
        const float dragMax = 0.01f;

        const float frictionMin = 0.2f;
        const float frictionMax =0.2f;

        const float stifnessMin = 0.05f;
        const float stifnessMax = 0.15f;

        const float gravityMin = 1.2f;
        const float gravityMax = 1.2f;

        const float weightMin = 0.5f;
        const float weightMax = 0.8f;

        //const float collisiobPower = 0.045f;
        const float collisiobPower = 0.06f;
        public static void Init()
        {
            csgDict = new Dictionary<string, ClothSettingGroup>();

            ClothSetting aEjacSettings = new ClothSetting()
            {
                compression = 1f,
                drag = 0.1f,
                friction = 0.5f,
                gravity = 0.8f,
                weight = 1.2f,
                scale = 0.6f,
                staticMulti = 2f,
                stiffness = 0.5f
            };
            ClothSetting aSecondLowerSettings = new ClothSetting()
            {
                compression = 0.3f,
                drag = dragMin,
                friction = frictionMin,
                gravity = gravityMin,
                weight = weightMin,
 //               scale = 0.5f,
                staticMulti = staticMin,
                stiffness = stifnessMin
            };
            ClothSetting aSecondUpperSettings = new ClothSetting()
            {
                compression = 0.7f,
                drag = dragMax,
                friction = frictionMax,
                gravity = gravityMax,
                weight = weightMax,
   //             scale = 0.8f,
                staticMulti = staticMax,
                stiffness = stifnessMax
            };

            ClothSetting bEjacSettings = new ClothSetting()
            {
                compression = 1f,
                drag = 0.1f,
                friction = 0.5f,
                gravity = 0.8f,
                weight = 0.9f,
                scale = 0.8f,
                staticMulti = 2f,
                stiffness = 0.5f
            };
            ClothSetting bSecondLowerSettings = new ClothSetting()
            {
                compression = 0.3f,
                drag = dragMin,
                friction = frictionMin,
                gravity = gravityMin,
                weight = weightMin,
                //                scale = 0.5f,
                staticMulti = staticMin,
                stiffness = stifnessMin
            };
            ClothSetting bSecondUpperSettings = new ClothSetting()
            {
                compression = 0.7f,
                drag = dragMax,
                friction = frictionMax,
                gravity = gravityMax,
                weight = weightMax,
 //               scale = 0.8f,
                staticMulti = staticMax,
                stiffness = stifnessMax
            };

            ClothSetting cEjacSettings = new ClothSetting()
            {
                compression = 1f,
                drag = 0.1f,
                friction = 0.5f,
                gravity = 0.8f,
                weight = 1.2f,
                scale = 0.8f,
                staticMulti = 2f,
                stiffness = 0.5f
            };
            ClothSetting cSecondLowerSettings = new ClothSetting()
            {
                compression = 0.8f,
                drag = dragMin,
                friction = frictionMin,
                gravity = gravityMin,
                weight = weightMin,
 //               scale = 0f,
                staticMulti = staticMin,
                stiffness = stifnessMin
            };
            ClothSetting cSecondUpperSettings = new ClothSetting()
            {
                compression = 1f,
                drag = dragMax,
                friction = frictionMax,
                gravity = gravityMax,
                weight = weightMax,
 //               scale = 0f,
                staticMulti = staticMax,
                stiffness = stifnessMax
            };

            csgDict.Add("A", new ClothSettingGroup(-0.3f, 0.0075f, collisiobPower, aEjacSettings, aSecondLowerSettings, aSecondUpperSettings,Color.red));
            csgDict.Add("B", new ClothSettingGroup(-0.3f, 0.0075f, collisiobPower, bEjacSettings, bSecondLowerSettings, bSecondUpperSettings, Color.green));
            csgDict.Add("C", new ClothSettingGroup(-0.3f, 0.0075f, collisiobPower, cEjacSettings, cSecondLowerSettings, cSecondUpperSettings, Color.blue));

        }
        public static void SetImpactSettings(string ordType, JSONStorable simClothingJSON, JSONStorable materialClothingJSON)
        {
            csgDict[ordType].SetImpactSettings(simClothingJSON, materialClothingJSON);
        }

        public static void SetEjacSettings(string ordType, JSONStorable simClothingJSON, JSONStorable materialClothingJSON)
        {
            csgDict[ordType].SetEjacSettings(simClothingJSON, materialClothingJSON);
        }
        public static void SetChamberLoadSettings(string ordType, JSONStorable simClothingJSON, JSONStorable materialClothingJSON)
        {
            csgDict[ordType].SetChamberLoadSettings(simClothingJSON, materialClothingJSON);
        }
        public static void SetAlpha(string ordType, JSONStorable materialClothingJSON, float level)
        {
            csgDict[ordType].SetAlpha(materialClothingJSON, level);
        }

        private static Dictionary<string, ClothSettingGroup> csgDict;
    }

    public class ClothSettingGroup
    {
        Color rgbColor;
        public ClothSettingGroup(float _alpha, float _collisionRadius, float _collisionPower, ClothSetting _ejacSetting, ClothSetting _secondaryLowerSettings, ClothSetting _secondaryUpperSettings, Color _rgbColor)
        {
            alpha = _alpha;

            rgbColor = _rgbColor;

            collisionRadius = _collisionRadius;
            collisionPower = _collisionPower;
            ejacSetting = _ejacSetting;
            secondarySetting = new ClothSettingRange(_secondaryLowerSettings, _secondaryUpperSettings);
        }

        public void SetEjacSettings(JSONStorable simClothingJSON, JSONStorable materialClothingJSON)
        {
            JSONStorableFloat alphaJSF = materialClothingJSON.GetFloatJSONParam("Alpha Adjust");
            alphaJSF.val = alpha;
            JSONStorableFloat collisionRadiusJSF = simClothingJSON.GetFloatJSONParam("collisionRadius");
            collisionRadiusJSF.val = Globals.cumAssist.cumStringCollisionDistanceJSF.val;
            JSONStorableFloat collisionPowerJSF = simClothingJSON.GetFloatJSONParam("collisionPower");
            collisionPowerJSF.val = collisionPower;

            JSONStorableFloat iterationsJSF = simClothingJSON.GetFloatJSONParam("iterations");
            iterationsJSF.val = Globals.cumAssist.cumStringGPUIterationsJSF.val;

            ejacSetting.SetSettings(simClothingJSON);
        }

        public void SetChamberLoadSettings(JSONStorable simClothingJSON, JSONStorable materialClothingJSON)
        {
            JSONStorableFloat alphaJSF = materialClothingJSON.GetFloatJSONParam("Alpha Adjust");
            alphaJSF.val = -1f;

            JSONStorableFloat collisionRadiusJSF = simClothingJSON.GetFloatJSONParam("collisionRadius");
            collisionRadiusJSF.val = Globals.cumAssist.cumStringCollisionDistanceJSF.val;
            JSONStorableFloat collisionPowerJSF = simClothingJSON.GetFloatJSONParam("collisionPower");
            collisionPowerJSF.val = 1f;
            JSONStorableFloat iterationsJSF = simClothingJSON.GetFloatJSONParam("iterations");
            iterationsJSF.val = Globals.cumAssist.cumStringGPUIterationsJSF.val;

            ejacSetting.SetChamberLoadSettings(simClothingJSON);
        }

        public void SetImpactSettings(JSONStorable simClothingJSON, JSONStorable materialClothingJSON)
        {
            JSONStorableFloat alphaJSF = materialClothingJSON.GetFloatJSONParam("Alpha Adjust");
            alphaJSF.val = alpha;

            JSONStorableFloat diffuseTexOffsetJSF = materialClothingJSON.GetFloatJSONParam("Diffuse Texture Offset");
            diffuseTexOffsetJSF.val =1f;

            JSONStorableFloat collisionRadiusJSF = simClothingJSON.GetFloatJSONParam("collisionRadius");
            collisionRadiusJSF.val = Globals.cumAssist.cumStringCollisionDistanceJSF.val;
            JSONStorableFloat collisionPowerJSF = simClothingJSON.GetFloatJSONParam("collisionPower");
            collisionPowerJSF.val = collisionPower;
            JSONStorableFloat iterationsJSF = simClothingJSON.GetFloatJSONParam("iterations");
            iterationsJSF.val = Globals.cumAssist.cumStringGPUIterationsJSF.val;

            JSONStorableFloat stiffnessJSF = simClothingJSON.GetFloatJSONParam("stiffness");
            stiffnessJSF.val = secondarySetting.stiffness;
            JSONStorableFloat scaleJSF = simClothingJSON.GetFloatJSONParam("distanceScale");
  //          scaleJSF.val = secondarySetting.scale;
            JSONStorableFloat compressionJSF = simClothingJSON.GetFloatJSONParam("compressionResistance");
            compressionJSF.val = secondarySetting.compression ;
            JSONStorableFloat gravityJSF = simClothingJSON.GetFloatJSONParam("gravityMultiplier");
            gravityJSF.val = secondarySetting.gravity;
            JSONStorableFloat weightJSF = simClothingJSON.GetFloatJSONParam("weight");
            weightJSF.val = secondarySetting.weight;
            JSONStorableFloat dragJSF = simClothingJSON.GetFloatJSONParam("drag");
            dragJSF.val = secondarySetting.drag;

            JSONStorableFloat frictionJSF = simClothingJSON.GetFloatJSONParam("friction");            
            float frictionRange = Globals.cumAssist.maxCumStickinessJSF.val - Globals.cumAssist.minCumStickinessJSF.val;            
            frictionJSF.val = 0.1f +(Globals.cumAssist.minCumStickinessJSF.val +(frictionRange* Random.value))/10f;

            JSONStorableFloat staticMultiJSF = simClothingJSON.GetFloatJSONParam("staticMultiplier");
            staticMultiJSF.val = secondarySetting.staticMulti;
            JSONStorableFloat skinJointStrengthJSF = simClothingJSON.GetFloatJSONParam("jointStrength");
            skinJointStrengthJSF.val = 0f;
        }

        public void SetAlpha(JSONStorable materialClothingJSON, float level)
        {
            JSONStorableFloat alphaJSF = materialClothingJSON.GetFloatJSONParam("Alpha Adjust");
            alphaJSF.val = -1f+((1f+alpha)*level);
            
        }

        public float alpha { get; }
        public float collisionRadius { get; }
        public float collisionPower { get; }

        private ClothSetting ejacSetting;
        private ClothSettingRange secondarySetting;
    }

    public class ClothSettingRange
    {
        public ClothSettingRange(ClothSetting _lowerClothSetting, ClothSetting _upperClothSetting)
        {
            lowerClothSetting = _lowerClothSetting;
            upperClothSetting = _upperClothSetting;
//            System.Random random = new System.Random();
        }

        private float rangeValue(float lowerValue, float upperValue)
        {
             
            return Random.value * (upperValue - lowerValue) + lowerValue;

        }

        public float stiffness { get { return rangeValue(lowerClothSetting.stiffness, upperClothSetting.stiffness); } }
        public float scale { get { return rangeValue(lowerClothSetting.scale, upperClothSetting.scale); } }
        public float compression { get { return rangeValue(lowerClothSetting.compression, upperClothSetting.compression); } }
        public float gravity { get { return rangeValue(lowerClothSetting.gravity, upperClothSetting.gravity); } }
        public float weight { get { return rangeValue(lowerClothSetting.weight, upperClothSetting.weight); } }
        public float drag { get { return rangeValue(lowerClothSetting.drag, upperClothSetting.drag); } }
        public float friction { get { return rangeValue(lowerClothSetting.friction, upperClothSetting.friction); } }
        public float staticMulti { get { return rangeValue(lowerClothSetting.staticMulti, upperClothSetting.staticMulti); } }

//        private System.Random random;
        private ClothSetting lowerClothSetting;
        private ClothSetting upperClothSetting;
    }

    public class ClothSetting
    {
        public void SetSettings(JSONStorable simClothingJSON)
        {
            JSONStorableFloat stiffnessJSF = simClothingJSON.GetFloatJSONParam("stiffness");
            stiffnessJSF.val = stiffness;
            JSONStorableFloat scaleJSF = simClothingJSON.GetFloatJSONParam("distanceScale");
            scaleJSF.val = Random.Range(scale / 2f, scale);
 //           scaleJSF.val = scale;
            JSONStorableFloat compressionJSF = simClothingJSON.GetFloatJSONParam("compressionResistance");
            compressionJSF.val = Random.Range(0.1f, compression); ;
            JSONStorableFloat gravityJSF = simClothingJSON.GetFloatJSONParam("gravityMultiplier");
            gravityJSF.val = gravity;
            JSONStorableFloat weightJSF = simClothingJSON.GetFloatJSONParam("weight");
            weightJSF.val = weight;
            JSONStorableFloat dragJSF = simClothingJSON.GetFloatJSONParam("drag");
            dragJSF.val = drag;
            JSONStorableFloat frictionJSF = simClothingJSON.GetFloatJSONParam("friction");
            frictionJSF.val = friction;
            JSONStorableFloat staticMultiJSF = simClothingJSON.GetFloatJSONParam("staticMultiplier");
            staticMultiJSF.val = staticMulti;
            JSONStorableFloat skinJointStrengthJSF = simClothingJSON.GetFloatJSONParam("jointStrength");
            skinJointStrengthJSF.val = 0f;
        }

        public void SetChamberLoadSettings(JSONStorable simClothingJSON)
        {

            JSONStorableFloat stiffnessJSF = simClothingJSON.GetFloatJSONParam("stiffness");
            stiffnessJSF.val = stiffness;
            JSONStorableFloat scaleJSF = simClothingJSON.GetFloatJSONParam("distanceScale");
            scaleJSF.val = 1f;
            JSONStorableFloat compressionJSF = simClothingJSON.GetFloatJSONParam("compressionResistance");
            compressionJSF.val = compression;
            JSONStorableFloat gravityJSF = simClothingJSON.GetFloatJSONParam("gravityMultiplier");
            gravityJSF.val = 0f;
            JSONStorableFloat weightJSF = simClothingJSON.GetFloatJSONParam("weight");
            weightJSF.val = weight;
            JSONStorableFloat dragJSF = simClothingJSON.GetFloatJSONParam("drag");
            dragJSF.val = drag;
            JSONStorableFloat frictionJSF = simClothingJSON.GetFloatJSONParam("friction");
            frictionJSF.val = friction;
            JSONStorableFloat staticMultiJSF = simClothingJSON.GetFloatJSONParam("staticMultiplier");
            staticMultiJSF.val = staticMulti;
            JSONStorableFloat skinJointStrengthJSF = simClothingJSON.GetFloatJSONParam("jointStrength");
            skinJointStrengthJSF.val = 1f;
        }

        public float stiffness { get; set; }
        public float scale { get; set; }
        public float compression { get; set; }
        public float gravity { get; set; }
        public float weight { get; set; }
        public float drag { get; set; }
        public float friction { get; set; }
        public float staticMulti { get; set; }
    }
}
