using System.Collections;
using System.Collections.Generic;
using System;
using System.Linq;
using UnityEngine;



namespace JayJayWon
{
    public static class FireControl
    {
        public static void Init()
        {
            firedOrdnance = new List<Ordnance>();
            retiringOrdnance = new Dictionary<string, List<Ordnance>>();
            retiringOrdnanceRequestTime = new Dictionary<Ordnance, float> ();

            Globals.MVRS.StartCoroutine(CreateSphere((atomName) =>
            {
                sphereName = atomName;
                InitSphere();
            }));

            JSONStorable geometry = Globals.MVRS.containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;
            penisCurveMorph = morphControl.GetMorphByDisplayName("Curve Up_Down");
        }

        public static void AtomNameUpdate(string oldName, string newName)
        {
            if (oldName == sphereName) sphereName = newName;
        }

        public static void AtomRemovedUpdate(Atom atom)
        {
            if (atom.name == sphereName)
            {
                Globals.MVRS.StartCoroutine(CreateSphere((atomName) =>
                {
                    sphereName = atomName;
                    InitSphere();
                }));
            }
        }

        public static void OnDestroy()
        {
            Atom sphereAtom = Globals.MVRS.GetAtomById(sphereName);
            if (sphereAtom != null) sphereAtom.Remove();
        }

        private static void InitSphere()
        {
            Atom sphereAtom = Globals.MVRS.GetAtomById(sphereName);
            sphereAtom.hidden = true;

            JSONStorable atomControlJS = sphereAtom.GetStorableByID("AtomControl");
            JSONStorableBool collisionEnabledJSB = atomControlJS.GetBoolJSONParam("collisionEnabled");
            collisionEnabledJSB.val = false;

            JSONStorable controlJS = sphereAtom.GetStorableByID("control");
            JSONStorableBool interactableJSB = controlJS.GetBoolJSONParam("interactableInPlayMode");
            interactableJSB.val = false;

            JSONStorable atomScaleJS = sphereAtom.GetStorableByID("scale");
            JSONStorableFloat atomScaleJSF = atomScaleJS.GetFloatJSONParam("scale");
            atomScaleJSF.val = 0.07f;

            JSONStorable materialJS = sphereAtom.GetStorableByID("SphereMat");
            JSONStorableFloat alphaJSF = materialJS.GetFloatJSONParam("Alpha Adjust");
            alphaJSF.val = -1f;

            ResetSpherePosition();
        }

        private static void ResetSpherePosition()
        {
            Atom sphereAtom = Globals.MVRS.GetAtomById(sphereName);
            if (sphereAtom != null)
            {
                Rigidbody penisBaseRB = Globals.containingAtom.rigidbodies.First(rb => rb.name == "penisBaseControl");
                Rigidbody penisTipRB = Globals.containingAtom.rigidbodies.First(rb => rb.name == "penisTipControl");
                Rigidbody sphereControlRB = sphereAtom.rigidbodies.First(rb => rb.name == "control");

                sphereControlRB.transform.position = penisBaseRB.transform.position;
                sphereControlRB.transform.LookAt(penisTipRB.transform);
            }

        }
    
        private static void MoveSphere()
        {
            Atom sphereAtom = Globals.MVRS.GetAtomById(sphereName);
            if (sphereAtom != null)
            {
                Rigidbody sphereControlRB = sphereAtom.rigidbodies.First(rb => rb.name == "control");

                float timeSinceFireControlStart = Time.time - fireControlMoveStartTime;

                float ratio = Mathf.Clamp(timeSinceFireControlStart / fireSpeed, 0f, 1f);

                Rigidbody penisTipRB = Globals.containingAtom.rigidbodies.First(rb => rb.name == "penisTipControl");
                float distanceMovedByTarget = Vector3.Distance(penisTipRB.position + (penisTipRB.transform.forward * 0.25f), targetPosition);

                JSONStorable atomScaleJS = sphereAtom.GetStorableByID("scale");
                JSONStorableFloat atomScaleJSF = atomScaleJS.GetFloatJSONParam("scale");
                atomScaleJSF.val = Mathf.Clamp(distanceMovedByTarget * 1.5f, 0.07f, 0.3f);

                Vector3 currentPostion = startPosition + ((targetPosition - startPosition) * ratio);

                sphereControlRB.transform.position = currentPostion;
                if (ratio > 0.4f && !salvo.IsLaunched())
                {
                    salvo.SetLaunched(firedOrdnance);
                }
                if (ratio == 1f)
                {                  
                    atomScaleJSF.val = 0.07f;
                    if (Globals.cumAssist.autoFireEnabledJSB.val && !Globals.cumAssist.autoFireContinuousJSB.val) 
                    {
                        if (Globals.cumAssist.remainingAutoFireShots>0) Globals.cumAssist.remainingAutoFireShots += -1;
                        Globals.cumAssist.SetAutoFireCountButtonLabel();
                    }
                    fireControlActive = false;
                    salvo.SetUnloaded();
                    ResetSpherePosition();
                }
            }
        }


        public static void FireSalvo()
        {
            if (!fireControlActive)
            {
                fireControlActive = true;
                sphereMovementStarted = false;
                chamberLoading = false;
            }
        }

        public static void Update()
        {
            if (fireControlActive)
            {
                
                if (!chamberLoading)
                {
                    salvo = OrdnanceDepot.CheckOutSalvo();
                    fireControlStartTime = Time.time;
                    chamberLoading = true;
                    salvo.ChamberLoad();
                    Rigidbody penisTipRB = Globals.containingAtom.rigidbodies.First(rb => rb.name == "penisTipControl");
                    targetPosition = penisTipRB.transform.position + (penisTipRB.transform.forward * 0.25f);
                    Rigidbody penisBaseRB = Globals.containingAtom.rigidbodies.First(rb => rb.name == "penisBaseControl");
                    startPosition = penisBaseRB.transform.position;
                }
                if (salvo.IsLoaded())
                {
                    if (!sphereMovementStarted)
                    {
                        fireControlMoveStartTime = Time.time;
                        sphereMovementStarted = true;
                        fireSpeed = UnityEngine.Random.Range(0.1f, 0.2f);
                    }
                    MoveSphere();
                }
                else
                {
                    Atom sphereAtom = Globals.MVRS.GetAtomById(sphereName);
                    Rigidbody sphereControlRB = sphereAtom.rigidbodies.First(rb => rb.name == "control");
                    sphereControlRB.transform.position = startPosition;
                }
                if (Globals.cumAssist.enableCockTwitchJSB.val) PenisTwitch();
            }
            else
            {
                ResetSpherePosition();
                if (Globals.cumAssist.autoFireEnabledJSB.val)
                {
                    if (Globals.cumAssist.autoFireContinuousJSB.val || Globals.cumAssist.remainingAutoFireShots > 0)
                    {
                        if (lastAutoFireTime + nextAutoFireInterval < Time.time)
                        {
                            lastAutoFireTime = Time.time;
                            nextAutoFireInterval = UnityEngine.Random.Range(Globals.cumAssist.minFireIntervalJSF.val, Globals.cumAssist.maxFireIntervalJSF.val);
                            twitchRadomizerMultiplier = UnityEngine.Random.Range(1f,6f);
                            FireSalvo();
                        }
                    }

                }
            }
            CheckOrdnanceReturn();
        }

        private static float twitchRadomizerMultiplier = 3f;

        private static void PenisTwitch()
        {
            float time = Time.time - fireControlStartTime;

            float twitchDuration = 0.3f;
            float twitchPeriod = twitchDuration / 2f;
            if (time < twitchPeriod)
            {
                penisCurveMorph.SetValue((time * twitchRadomizerMultiplier));
            }
            else if (time < twitchDuration)
            {
                penisCurveMorph.SetValue(twitchRadomizerMultiplier*(twitchPeriod - (time- twitchPeriod)));
            }
        }

        private static IEnumerator CreateSphere(Action<string> onSphereCreated)
        {
            List<Atom> atoms = SuperController.singleton.GetAtoms();
            List<string> atomNames = new List<string>();
            foreach (Atom tempAtom in atoms) atomNames.Add(tempAtom.uid);
            string atomName = "JayJayWon.CumAssist.Sphere";

            if (atomNames.Contains(atomName) && SuperController.singleton.GetAtomByUid(atomName).type != "Sphere")
            {
                for (int i = 2; i < 1000; i++)
                {
                    if (!atomNames.Contains(atomName + "#" + i.ToString()))
                    {
                        atomName = atomName + "#" + i.ToString();
                        break;
                    }
                }
            }

            if (!atomNames.Contains(atomName)) yield return SuperController.singleton.AddAtomByType("Sphere", atomName);

            onSphereCreated(atomName);
        }

        public static void RequestOrdnanceReturn(string requestedOrdnanceType, int quantityRequested)
        {
            int foundOrdnance = 0;
            List<Ordnance> ordnanceToReturn = new List<Ordnance>();
            foreach (Ordnance ordnance in firedOrdnance)
            {
                string ordnanceType = ordnance.GetOrdnanceType();
                if (foundOrdnance < quantityRequested && ordnanceType == requestedOrdnanceType) 
                {
                    ordnanceToReturn.Add(ordnance);
                    if (!retiringOrdnance.ContainsKey(ordnanceType)) retiringOrdnance.Add(ordnanceType, new List<Ordnance>());
                    retiringOrdnance[ordnanceType].Add(ordnance);
                    if (!retiringOrdnanceRequestTime.ContainsKey(ordnance)) retiringOrdnanceRequestTime.Add(ordnance, Time.time);
                    else retiringOrdnanceRequestTime[ordnance] = Time.time;
                    foundOrdnance++;
                }
            }

            foreach (Ordnance ordnance in ordnanceToReturn) firedOrdnance.Remove(ordnance);
        }

        public static void RequestImmediateOrdnanceReturn(string requestedOrdnanceType)
        {
            if (retiringOrdnance.ContainsKey(requestedOrdnanceType) && retiringOrdnance[requestedOrdnanceType].Count > 0)
            {
                retiringOrdnanceRequestTime[retiringOrdnance[requestedOrdnanceType].First()] = 0f;
                CheckOrdnanceReturn();
            }
            else
            {
                List<Ordnance> ordList = new List<Ordnance>();
                Ordnance ordnance = firedOrdnance.First(s => s.GetOrdnanceType() == requestedOrdnanceType);
                ordnance.TurnOff();
                ordList.Add(ordnance);
                OrdnanceDepot.ReturnOrdnance(requestedOrdnanceType, ordList);
            }
        }
        public static void ClearAllOrdnance()
        {
            List<Ordnance> ordAList = new List<Ordnance>();
            List<Ordnance> ordBList = new List<Ordnance>();
            List<Ordnance> ordCList = new List<Ordnance>();

            foreach (Ordnance ordnance in firedOrdnance)
            {
                if (ordnance.GetOrdnanceType() == "A") ordAList.Add(ordnance);
                if (ordnance.GetOrdnanceType() == "B") ordBList.Add(ordnance);
                if (ordnance.GetOrdnanceType() == "C") ordCList.Add(ordnance);
                ordnance.TurnOff();
            }
            firedOrdnance.Clear();

            foreach (string ordnanceType in retiringOrdnance.Keys.ToList())
            {
                if (retiringOrdnance[ordnanceType] != null)
                {
                    foreach (Ordnance ordnance in retiringOrdnance[ordnanceType])
                    {
                        if (ordnance.GetOrdnanceType() == "A") ordAList.Add(ordnance);
                        if (ordnance.GetOrdnanceType() == "B") ordBList.Add(ordnance);
                        if (ordnance.GetOrdnanceType() == "C") ordCList.Add(ordnance);
                        ordnance.TurnOff();
                    }
                    retiringOrdnance[ordnanceType].Clear();

                }
                retiringOrdnanceRequestTime.Clear();
            }
            if (ordAList.Count > 0) OrdnanceDepot.ReturnOrdnance("A", ordAList);
            if (ordBList.Count > 0) OrdnanceDepot.ReturnOrdnance("B", ordBList);
            if (ordCList.Count > 0) OrdnanceDepot.ReturnOrdnance("C", ordCList);

        }


        private static void CheckOrdnanceReturn()
        {
            foreach (string ordnanceType in retiringOrdnance.Keys.ToList())
            {
                if (retiringOrdnance[ordnanceType] != null)
                {
                    List<Ordnance> ordnancetoReturn = new List<Ordnance>();
                    foreach (Ordnance ordnance in retiringOrdnance[ordnanceType])
                    {
                        float time = Time.time - retiringOrdnanceRequestTime[ordnance];
                        if ( time> ordnanceRetireTime)
                        {
                            ordnancetoReturn.Add(ordnance);
                            ordnance.TurnOff();
                        }
                        else ordnance.SetAlpha(1 - (time / ordnanceRetireTime));

                    }
                    
                    if (ordnancetoReturn.Count > 0)
                    {
                        foreach (Ordnance ordnance in ordnancetoReturn)
                        {
                            retiringOrdnance[ordnanceType].Remove(ordnance);
                            retiringOrdnanceRequestTime.Remove(ordnance);
                        }
                        OrdnanceDepot.ReturnOrdnance(ordnanceType, ordnancetoReturn);
                    }
                }

            }


           

        }

        public static bool IsFireControlActive() { return fireControlActive; }

        private const float ordnanceRetireTime = 3f;

        private static bool fireControlActive = false;
        private static bool sphereMovementStarted = false;
        private static bool chamberLoading = false;

        private static float fireControlMoveStartTime = 0f;
        private static float fireControlStartTime = 0f;
        private static float fireSpeed;
        private static Salvo salvo;

        private static List<Ordnance> firedOrdnance;

        private static Dictionary<string, List<Ordnance>> retiringOrdnance;
        private static Dictionary<Ordnance, float> retiringOrdnanceRequestTime;

        private static Vector3 targetPosition ;
        private static Vector3 startPosition ;

        private static string sphereName;

        private static DAZMorph penisCurveMorph ;

        private static float lastAutoFireTime =0f;
        private static float nextAutoFireInterval = -1f;

    }


}